<?php
/**
 * This file is part of PHPWord - A pure PHP library for reading and writing
 * word processing documents.
 *
 * PHPWord is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPWord/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPWord
 * @copyright   2010-2014 PHPWord contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpWord\Element;

use PhpOffice\PhpWord\Exception\InvalidObjectException;
use PhpOffice\PhpWord\Settings;
use PhpOffice\PhpWord\Shared\ZipArchive;
use PhpOffice\PhpWord\Style\Image as ImageStyle;

/**
 * OLEObject element
 */
class OLEObject extends AbstractElement
{
    /**
     * File source type constants
     */
    const SOURCE_LOCAL = 'local'; // Local file
    const SOURCE_ARCHIVE = 'archive'; // File in archives zip://$archive#$image

    /**
     * Ole-Object Src
     *
     * @var string
     */
    private $source;

    /**
     * Ole-Decode src
     *
     * @var string
     */
    private $fileSource = null;

    /**
     * Source type: local|archive
     *
     * @var string
     */
    private $sourceType;

    /**
     * Image Style
     *
     * @var \PhpOffice\PhpWord\Style\Image
     */
    private $style;

    /**
     * Icon
     *
     * @var string
     */
    private $icon;

    /**
     * Image Relation ID
     *
     * @var int
     */
    private $imageRelationId;

    /**
     * attachment filename
     *
     * @var string
     */
    private $filename = null;

    /**
     * Has media relation flag; true for Link, Image, and Object
     *
     * @var bool
     */
    protected $mediaRelation = true;

    /**
     * Create a new Ole-Object Element
     *
     * @param string $source
     * @param mixed $style
     * @throws \PhpOffice\PhpWord\Exception\InvalidObjectException
     */
    public function __construct($source, $style = null)
    {
        //$supportedTypes = array('xls', 'doc', 'ppt', 'xlsx', 'docx', 'pptx');

        $this->setSourceType($source);

        $this->readOLEStream();

        $this->style = $this->setNewStyle(new ImageStyle(), $style, true);
        if (file_exists($this->fileSource)) {
            return $this;
        } else {
            throw new InvalidObjectException();
        }
    }

    /**
     * unlink temp file in destructor
     */
    public function __destruct()
    {
        if (file_exists($this->fileSource)) {
            unlink($this->fileSource);
        }
    }

    /**
     * Get mapping file icon from it's original extension
     *
     * @param string $ext
     * @return string $mapping_ext
     */
    public function getExtMappingIcon($ext)
    {
        $ext_mapping = array(
            "jpg" => explode(" ", "bmp ico tif tiff ufo raw arw srf sr2 dcr k25 kdc cr2 crw nef mrw ptx pef raf 3fr erf mef mos orf rw2 dng x3f jpg jpe jpeg png"),
            "htm" => explode(" ", "htm html"),
            "avi" => explode(" ", "avi 3gp 3g2 asf dat divx dvr-ms m2t m2ts m4v mkv mp4 mts mov qt tp trp ts vob wmv xvid ac3 amr rm rmvb ifo mpeg mpg mpe m1v m2v mpeg1 mpeg2 mpeg4 ogv webm"),
            "mp3" => explode(" ", "mp3 aac flac m4a m4b aif ogg pcm wav cda mid mp2 mka mpc ape ra ac3 dts"),
            "iso" => explode(" ", "iso bin img mds nrg daa"),
            "doc" => explode(" ", "doc docx wri rtf"),
            "xls" => explode(" ", "xls xla xlb xlc xld xlk xll xlm xlt xlv xlw xlsx xlsm xlsb xltm xlam"),
            "ppt" => explode(" ", "ppt pptx pps ppsx"),
            "acc" => explode(" ", "acc ade adp adn accdr accdb accdt mdb mda mdn mdt mdw mdf mde accde mam maq mar mat maf"),
            "swf" => explode(" ", "swf flv f4v"),
            "zip" => explode(" ", "zip tar 7z bz2 gz tgz tbz"),
            "ttf" => explode(" ", "ttf ttc otf"),
            "js" => explode(" ", "js css actproj ad akp applescript as asax asc ascx asm asmx asp aspx asr c cc php jsx xml xhtml mhtml cpp cs cxx")
        );

        $ext = strtolower($ext);
        foreach ($ext_mapping as $key => $value) {
            if (in_array($ext, $value)) {
                return $key;
            }
        }
        return $ext;
    }

    /**
     * Get object source
     *
     * @return string
     */
    public function getSource()
    {
        return $this->source;
    }

    /**
     * Get object style
     *
     * @return \PhpOffice\PhpWord\Style\Image
     */
    public function getStyle()
    {
        return $this->style;
    }

    /**
     * Get object icon
     *
     * @return string
     */
    public function getIcon()
    {
        return $this->icon;
    }

    /**
     * Get image relation ID
     *
     * @return int
     */
    public function getImageRelationId()
    {
        return $this->imageRelationId;
    }

    /**
     * Set Image Relation ID.
     *
     * @param int $rId
     * @return void
     */
    public function setImageRelationId($rId)
    {
        $this->imageRelationId = $rId;
    }

    /**
     * Get Object ID
     *
     * @return int
     * @deprecated 0.10.0
     * @codeCoverageIgnore
     */
    public function getObjectId()
    {
        return $this->relationId + 1325353440;
    }

    /**
     * Set Object ID
     *
     * @param int $objId
     * @deprecated 0.10.0
     * @codeCoverageIgnore
     */
    public function setObjectId($objId)
    {
        $this->relationId = $objId;
    }

    /**
     * Set source type.
     *
     * @param string $source
     * @return void
     */
    private function setSourceType($source)
    {
        $this->source = $source;
        if (strpos($source, 'zip://') !== false) {
            $this->sourceType = self::SOURCE_ARCHIVE;
        } else {
            $this->sourceType = self::SOURCE_LOCAL;
        }
    }

    private function updateSourceIcon($source)
    {
        $pathInfo = pathinfo($source);
        $ext = $pathInfo['extension'];
        if (strlen($ext) == 4 && strtolower(substr($ext, -1)) == 'x') {
            $ext = substr($ext, 0, -1);
        }
        $ext = $this->getExtMappingIcon($ext);

        if (file_exists(__DIR__ . "/../resources/{$ext}.png")) {
            $this->icon = realpath(__DIR__ . "/../resources/{$ext}.png");
        } else {
            $this->icon = realpath(__DIR__ . "/../resources/xxx.png");
        }
    }

    /**
     * decode OLE stream to get file content and filename
     */
    private function readOLEStream()
    {
        global $synoTempPath;
        $tempFilename = '';
        $source = $this->source;

        // make a temp file for archive source
        if ($this->sourceType == self::SOURCE_ARCHIVE) {
            $source = substr($source, 6);
            list($zipFilename, $objFilename) = explode('#', $source);

            $tempDir = empty($synoTempPath) ? Settings::getTempDir() : $synoTempPath;
            $tempFilename = tempnam($tempDir, 'PHPWordObject');
            if (false === $tempFilename) {
                throw new CreateTemporaryFileException();
            }

            $zip = new ZipArchive();
            if ($zip->open($zipFilename) !== false) {
                if ($zip->locateName($objFilename) !== false) {
                    $objContent = $zip->getFromName($objFilename);
                    if ($objContent !== false) {
                        file_put_contents($tempFilename, $objContent);
                        $source = $tempFilename;
                    }
                }
                $zip->close();
            }
        }

        $ole = new \PhpOffice\PhpWord\Shared\OLERead();
        $ole->read($source);

        $stream_idx = -1;
        foreach($ole->props as $idx => $stream) {
            if (preg_match('/ole.*native/i', $stream['name'], $matches)) {
                $stream_idx = $idx;
                break;
            }
        }
        if ($stream_idx < 0) {
            $join_names = function($str, $stream) { return $str . ", " . $stream['name']; };
            echo "Error: failed to get OLE stream index from entries:". array_reduce(array_values($ole->props), $join_names, '') ."\n";
            return '';
        }

        echo "=== Converting OLE to file ===\n";
        $fidx = 0;
        $data = $ole->getStream($stream_idx);

        // reading length of pre/post and file stream
        $arr = unpack('Vlen', substr($data, $fidx, 4));
        $fidx += 4;

        // skip 2 bytes of \x02\x00
        $fidx += 2;

        // reading filename
        $filename = substr($data, $fidx, strpos(substr($data, $fidx), "\x00"));
        $fidx += strlen($filename) + 1;
        $this->filename = $filename;
        $this->updateSourceIcon($this->filename);
        echo "name:\t$filename\n";

        // reading file path
        $path = substr($data, $fidx, strpos(substr($data, $fidx), "\x00"));
        $fidx += strlen($path) + 1;
        echo "path:\t$path\n";

        // skip 4 bytes of \x00\x00\x03\x00
        $fidx += 4;

        // reading tmp path length
        $arr = unpack('Vlen', substr($data, $fidx, 4));
        $fidx += 4;

        // reading tmp path
        $path = substr($data, $fidx, strpos(substr($data, $fidx), "\x00"));
        $fidx += strlen($path) + 1;
        echo "temp:\t$path\n";

        // reading file size
        $arr = unpack('Vsize', substr($data, $fidx, 4));
        $fsize = $arr['size'];
        $fidx += 4;
        echo "size:\t" . $fsize . "\n";

        $this->generateFileSource(substr($data, $fidx, $fsize));

        if (!empty($tempFilename) && file_exists($tempFilename)) {
            unlink($tempFilename);
        }
    }

    /**
     * Create a file for holding file source
     */
    private function generateFileSource($stream)
    {
        global $synoTempPath;

        $tempDir = empty($synoTempPath) ? Settings::getTempDir() : $synoTempPath;
        $tempFilename = tempnam($tempDir, 'PHPWordObject');
        if (false === $tempFilename) {
            throw new CreateTemporaryFileException();
        }

        $this->fileSource = $tempFilename;

        file_put_contents($this->fileSource, $stream);
    }

    /**
     * Get image string data
     *
     * @return string|null
     */
    public function getObjectStringData()
    {
        if ($this->fileSource === null) {
            $this->readOLEStream();
        }
        return file_get_contents($this->fileSource);
    }

    /**
     * Get attachment filename
     *
     * @return string|null
     */
    public function getFilename()
    {
        if ($this->filename === null) {
            $this->readOLEStream();
        }
        return $this->filename;
    }
}
