<?php
/**
 * This file is part of PHPWord - A pure PHP library for reading and writing
 * word processing documents.
 *
 * PHPWord is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPWord/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPWord
 * @copyright   2010-2014 PHPWord contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpWord\Shared;

use PhpOffice\PhpWord\PhpWord;
use PhpOffice\PhpWord\Element\AbstractContainer;
use PhpOffice\PhpWord\Style\ListItem as ListItemStyle;
use PhpOffice\PhpWord\Style\Paragraph as ParagraphStyle;
use PhpOffice\PhpWord\Shared\Converter;

/**
 * Common Html functions
 *
 * @SuppressWarnings(PHPMD.UnusedPrivateMethod) For readWPNode
 */
class Html
{
    /**
     * Add HTML parts.
     *
     * Note: $stylesheet parameter is removed to avoid PHPMD error for unused parameter
     *
     * @param \PhpOffice\PhpWord\Element\AbstractContainer $element Where the parts need to be added
     * @param string $html The code to parse
     * @param bool $fullHTML If it's a full HTML, no need to add 'body' tag
     * @return void
     */
    public static function addHtml($element, $html, $fullHTML = false)
    {
        /*
         * @todo parse $stylesheet for default styles.  Should result in an array based on id, class and element,
         * which could be applied when such an element occurs in the parseNode function.
         */

        // Preprocess: remove all line ends, decode HTML entity,
        // fix ampersand and angle brackets and add body tag for HTML fragments
        $html = str_replace(array("\n", "\r"), '', $html);
        $html = str_replace(array('&lt;', '&gt;', '&amp;', '&quot;', '&nbsp;'), array('_lt_', '_gt_', '_amp_', '_quot_', '_nbsp_'), $html);
        $html = html_entity_decode($html, ENT_QUOTES, 'UTF-8');
        $html = str_replace('&', '&amp;', $html);
        $html = str_replace(array('_lt_', '_gt_', '_amp_', '_quot_', '_nbsp_'), array('&lt;', '&gt;', '&amp;', '&quot;', '&nbsp;'), $html);

        if ($fullHTML === false) {
            $html = '<body>' . $html . '</body>';
        }

        // Load DOM
        $dom = new \DOMDocument();
        $dom->preserveWhiteSpace = true;
        libxml_use_internal_errors(true); // suppress warnings
        $dom->loadHTML(mb_convert_encoding($html, 'HTML-ENTITIES', 'UTF-8'));
        libxml_clear_errors();
        $node = $dom->getElementsByTagName('body');

        self::parseNode($node->item(0), $element);
    }

    /**
     * parse Inline style of a node
     *
     * @param \DOMNode $node Node to check on attributes and to compile a style array
     * @param array $styles is supplied, the inline style attributes are added to the already existing style
     * @return array
     */
    protected static function parseInlineStyle($node, $styles = array())
    {
        if ($node->nodeType == XML_ELEMENT_NODE) {
            $attributes = $node->attributes; // get all the attributes(eg: id, class)

            foreach ($attributes as $attribute) {
                switch ($attribute->name) {
                    case 'style':
                        $styles = self::parseStyle($attribute, $styles);
                        break;
                    case 'synohead':
                        $styles[$attribute->name] = $attribute->value;
                        break;
                }
            }
        }

        return $styles;
    }

    /**
     * Parse a node and add a corresponding element to the parent element.
     *
     * @param \DOMNode $node node to parse
     * @param \PhpOffice\PhpWord\Element\AbstractContainer $element object to add an element corresponding with the node
     * @param array $styles Array with all styles
     * @param array $data Array to transport data to a next level in the DOM tree, for example level of listitems
     * @return void
     */
    protected static function parseNode($node, $element, $styles = array(), $data = array(), $tag_level = 0)
    {
        // Populate styles array
        $styleTypes = array('font', 'paragraph', 'list');
        foreach ($styleTypes as $styleType) {
            if (!isset($styles[$styleType])) {
                $styles[$styleType] = array();
            }
        }

        // Node mapping table
        $nodes = array(
                                 // $method     $node   $element    $styles     $data   $argument1      $argument2
            'p'         => array('Paragraph',   $node,  $element,   $styles,    null,   null,           null),
            'div'       => array('Div',         $node,  $element,   $styles,    null,   null,           null),
            'h1'        => array('Heading',     $node,  $element,   $styles,    null,   'Heading1',     1),
            'h2'        => array('Heading',     $node,  $element,   $styles,    null,   'Heading2',     2),
            'h3'        => array('Heading',     $node,  $element,   $styles,    null,   'Heading3',     3),
            'h4'        => array('Heading',     $node,  $element,   $styles,    null,   'Heading4',     4),
            'h5'        => array('Heading',     $node,  $element,   $styles,    null,   'Heading5',     5),
            'h6'        => array('Heading',     $node,  $element,   $styles,    null,   'Heading6',     6),
            '#text'     => array('Text',        $node,  $element,   $styles,    null,   null,           null),
                                                        // prosemirror 3df7cc2e5f137372c3320635b4af065041bd556c
                                                        // chart is held in span instead of div now
            'span'      => array('Span',        $node,  $element,   $styles,    null,   null,           null), // parse inline style on span
            'strong'    => array('Property',    null,   null,       $styles,    null,   'bold',         true),
            'b'         => array('Property',    null,   null,       $styles,    null,   'bold',         true), // set text style to bold
            'em'        => array('Property',    null,   null,       $styles,    null,   'italic',       true),
            'i'         => array('Property',    null,   null,       $styles,    null,   'italic',       true), // set text style to italic
            'u'         => array('Property',    null,   null,       $styles,    null,   'underline',    'single'), // set text underline
            'sup'       => array('Property',    null,   null,       $styles,    null,   'superScript',  true),
            'sub'       => array('Property',    null,   null,       $styles,    null,   'subScript',    true),
            's'         => array('Property',    null,   null,       $styles,    null,   'strikethrough',true),
            'strike'    => array('Property',    null,   null,       $styles,    null,   'strikethrough',true), // set text strike
            'table'     => array('Table',       $node,  $element,   $styles,    null,   'addTable',     static::getTableStyle()),
            'tr'        => array('Table',       $node,  $element,   $styles,    null,   'addRow',       true),
            'td'        => array('Table',       $node,  $element,   $styles,    null,   'addCell',      true),
            'ul'        => array('List',        null,   $element,   $styles,    $data,  ListItemStyle::TYPE_BULLET_FILLED,              null),
            'ol'        => array('List',        null,   $element,   $styles,    $data,  ListItemStyle::TYPE_NUMBER,              null),
            'li'        => array('ListItem',    $node,  $element,   $styles,    $data,  null,           null),
                                                                    // pass $styles to handle comments
            'img'       => array('Image',       $node,  $element,   $styles,    null,   null,           null), // add image
            'a'         => array('Anchor',      $node,  $element,   $styles,    null,   null,           null), // add link
            'hr'        => array('Line',        $node,  $element,   null,       null,   null,           null), // add horizontal line
            'input'     => array('Input',       $node,  $element,   null,       null,   null,           null), // add checkbox
            'nav'       => array('Nav',         null,   $element,   null,       $data,  null,           null), // parse table of contents
            'del'       => array('Revision',    $node,  null,       $styles,    null,   null,           null), // revision: deletion
            'ins'       => array('Revision',    $node,  null,       $styles,    null,   null,           null), // revision: insertion

            'header'    => array('Header',      $node,  $element,   $styles,    null,   null,           null), // Header
            'footer'    => array('Footer',      $node,  $element,   $styles,    null,   null,           null), // Footer
        );

        $newElement = null;
        $keys = array('node', 'element', 'styles', 'data', 'argument1', 'argument2');
        if (isset($nodes[$node->nodeName])) {
            // Execute method based on node mapping table and return $newElement or null
            // Arguments are passed by reference
            $arguments = array();
            $args = array();
            list($method, $args[0], $args[1], $args[2], $args[3], $args[4], $args[5]) = $nodes[$node->nodeName];
            for ($i = 0; $i <= 5; $i++) {
                if ($args[$i] !== null) {
                    $arguments[$keys[$i]] = &$args[$i];
                }
            }
            $method = "parse{$method}";
            $newElement = call_user_func_array(array(static::class, $method), $arguments);

            // Retrieve back variables from arguments
            foreach ($keys as $key) {
                if (array_key_exists($key, $arguments)) {
                    $$key = $arguments[$key];
                }
            }
        }

        if ($newElement === null) {
            $newElement = $element;
        }

        self::parseChildNodes($node, $newElement, $styles, $data, $tag_level + 1);
    }

    /**
     * Get table style for UI purpose, for child to overwrite
     *
     * @return array
     */
    protected static function getTableStyle()
    {
        return array('borderSize' => 6, 'borderColor' => '999999', 'width' => 100 * 50, "unit" => "pct");
    }

    /**
     * Parse child nodes.
     *
     * @param \DOMNode $node
     * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
     * @param array $styles
     * @param array $data
     * @return void
     */
    private static function parseChildNodes($node, $element, $styles, $data, $tag_level)
    {
        $table_container = array('PhpOffice\PhpWord\Element\Table', 'PhpOffice\PhpWord\Element\Row', 'PhpOffice\PhpWord\Element\Cell');
        $cNodes = $node->childNodes;
        if (!empty($cNodes)) {
            foreach ($cNodes as $cNode) {
                if ($element instanceof AbstractContainer) {
                    self::parseNode($cNode, $element, $styles, $data, $tag_level);
                } else if (in_array(get_class($element), $table_container)) {
                    self::parseNode($cNode, $element, $styles, $data, $tag_level);
                }
            }
        }
    }

    /**
     * Parse paragraph node
     *
     * @param \DOMNode $node
     * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
     * @param array &$styles
     * @return \PhpOffice\PhpWord\Element\TextRun
     */
    protected static function parseParagraph($node, $element, &$styles)
    {
        $styles['paragraph'] = static::parseInlineStyle($node, $styles['paragraph']);
        $newElement = $element->addTextRun($styles['paragraph']);

        return $newElement;
    }

    /**
     * Parse div node, do nothing, for child to overwrite
     *
     * @param \DOMNode $node
     * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
     * @param array &$styles
     * @return \PhpOffice\PhpWord\Element
     */
    protected static function parseDiv($node, $element, &$styles)
    {
        return $element;
    }

    /**
     * Parse heading node
     *
     * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
     * @param array &$styles
     * @param string $argument1 Name of heading style
     * @return \PhpOffice\PhpWord\Element\TextRun
     *
     * @todo Think of a clever way of defining header styles, now it is only based on the assumption, that
     * Heading1 - Heading6 are already defined somewhere
     */
    private static function parseHeading($node, $element, &$styles, $argument1, $argument2)
    {
        $pStyle = static::parseInlineStyle($node, $styles);
        $styles['paragraph'] = $argument1;
        $phpword = $element->getPhpWord();

        if ($phpword instanceof PhpWord) {
            $newElement = $element->addTitleRun($styles['paragraph'], $argument2, $pStyle['synohead'], $phpword);
        } else {
            $newElement = $element->addTextRun($styles['paragraph']);
        }

        return $newElement;
    }

    /**
     * Parse text node
     *
     * @param \DOMNode $node
     * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
     * @param array &$styles
     * @return null
     */
    private static function parseText($node, $element, &$styles)
    {
        $styles['font'] = static::parseInlineStyle($node, $styles['font']);

        // Commented as source of bug #257. `method_exists` doesn't seems to work properly in this case.
        // @todo Find better error checking for this one
        // if (method_exists($element, 'addText')) {
            $element->addText(htmlspecialchars($node->nodeValue), $styles['font'], $styles['paragraph']);
        // }

        return null;
    }

    /**
     * Parse property node
     *
     * @param array &$styles
     * @param string $argument1 Style name
     * @param string $argument2 Style value
     * @return null
     */
    private static function parseProperty(&$styles, $argument1, $argument2)
    {
        $styles['font'][$argument1] = $argument2;

        return null;
    }

    /**
     * Parse table node
     *
     * @param \DOMNode $node
     * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
     * @param array &$styles
     * @param string $argument1 Method name
     * @return \PhpOffice\PhpWord\Element\AbstractContainer $element
     *
     * @todo As soon as TableItem, RowItem and CellItem support relative width and height
     */
    private static function parseTable($node, $element, &$styles, $argument1, $argument2)
    {
        $styles['paragraph'] = static::parseInlineStyle($node, $styles['paragraph']);

        if ($argument1 === 'addTable' && !empty($argument2)) {
            if (array_key_exists('width', $styles['paragraph'])) {
                $argument2['width'] = $styles['paragraph']['width'] * 20;
                $argument2['unit'] = 'dxa';
            }
            $newElement = $element->$argument1($argument2);
        } else {
            $newElement = $element->$argument1();
        }

        // $attributes = $node->attributes;
        // if ($attributes->getNamedItem('width') !== null) {
            // $newElement->setWidth($attributes->getNamedItem('width')->value);
        // }

        // if ($attributes->getNamedItem('height') !== null) {
            // $newElement->setHeight($attributes->getNamedItem('height')->value);
        // }
        // if ($attributes->getNamedItem('width') !== null) {
            // $newElement=$element->addCell($width=$attributes->getNamedItem('width')->value);
        // }

        return $newElement;
    }

    /**
     * Parse list node
     *
     * @param array &$styles
     * @param array &$data
     * @param string $argument1 List type
     * @return null
     */
    private static function parseList($element, &$styles, &$data, $argument1)
    {
        static $listCount = 0;
        if (isset($data['listdepth'])) {
            $data['listdepth']++;
        } else {
            $data['listdepth'] = 0;
        }
        $styles['list']['listType'] = $argument1;
        $styles['list']['listId'] = $listCount++;

        if (!isset($data['list_container'])) {
            $data['list_container'] = $element;
        }

        return null;
    }

    /**
     * Parse list item node
     *
     * @param \DOMNode $node
     * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
     * @param array &$styles
     * @param array $data
     * @return null
     *
     * @todo This function is almost the same like `parseChildNodes`. Merged?
     * @todo As soon as ListItem inherits from AbstractContainer or TextRun delete parsing part of childNodes
     */
    private static function parseListItem($node, $element, &$styles, $data)
    {
        $depth = isset($data['listdepth']) ? $data['listdepth'] : 0;

        // retrieve syno-list-item marker
        if ( is_null($node->firstChild->firstChild) ) {
            $synoMarker = $node->firstChild;
        } else {
            $synoMarker = $node->firstChild->firstChild;
        }

        $markerStyle = null;
        if ( !is_null($synoMarker) && $synoMarker->nodeType == XML_ELEMENT_NODE ) {
            if ( is_null($synoMarker->attributes->getNamedItem('syno-list-item')) ) {
                if ( !is_null($node->firstChild->attributes->getNamedItem('textblock')) ) {
                    echo 'Warning: export ListItem marker failed!';
                }
            } else {
                $markerStyle = array('synoListItem' => static::parseInlineStyle($synoMarker));
            }
        }

        if (isset($data['list_container'])) {
            $newElement = $data['list_container']->addListItemRun($depth, $styles['list'], $markerStyle);
        } else {
            $newElement = $element->addListItemRun($depth, $styles['list'], $markerStyle);
        }

        return $newElement;
    }

    /**
     * Parse style
     *
     * @param \DOMAttr $attribute
     * @param array $styles
     * @return array
     */
    protected static function parseStyle($attribute, $styles)
    {
        /**
         * Transform the color format from 'rgb(r,g,b)' to '#rgb' without the leading '#'
         */
        $transformColorFormat = function ($color) {
            if (strpos($color, 'rgb') !== 0) {
                return $color;
            }

            /**
             * printf("%02x", n);
             */
            $dec2hex = function ($n) {
                return str_pad(dechex((int)$n), 2, '0', STR_PAD_LEFT);
            };

            // The color format may be 'rgb(r,g,b)' or 'rgba(r,g,b,a)'
            // Only the values of r, g, b are considered
            preg_match('/^rgba?\((\d+),\s+(\d+),\s+(\d+)[^)]*\)$/', $color, $matches);

            $r = $matches[1];
            $g = $matches[2];
            $b = $matches[3];
            return ($dec2hex($r) . $dec2hex($g) . $dec2hex($b));
        };

        $transformBorderStyle = function ($value, $default) {
            if ( empty($default) ) { return 'single'; } // TODO
            return $default;
        };

        $properties = explode(';', trim($attribute->value, " \t\n\r\0\x0B;"));
        foreach ($properties as $property) {
            list($cKey, $cValue) = explode(':', $property, 2);
            $cValue = trim($cValue);
            switch (trim($cKey)) {
                case 'text-decoration':
                    switch ($cValue) {
                        case 'underline':
                            $styles['underline'] = 'single';
                            break;
                        case 'line-through':
                            $styles['strikethrough'] = true;
                            break;
                    }
                    break;
                case 'text-align':
                    $styles['align'] = $cValue;
                    break;
                case 'color':
                    $styles['color'] = $transformColorFormat(trim($cValue, "#"));
                    break;
                case 'background-color':
                    $styles['bgColor'] = $transformColorFormat(trim($cValue, "#"));
                    break;
                case 'font-family': // add text font-family
                    $styles['name'] = trim($cValue);
                    break;
                case 'padding-left': // add indent style
                    $styles['indentation'] = array('left' => intval($cValue) * 8);
                    break;
                case 'width':
                    $styles['width'] = $cValue;
                    break;
                case 'height':
                    $styles['height'] = $cValue;
                    break;
                case 'font-size':
                    $styles['size'] = floatval($cValue);
                    break;
                case 'font-weight':
                    if ( $cValue === 'bold' ) {
                        $styles['bold'] = true;
                    }
                    break;
                case 'font-style':
                    if ( $cValue === 'italic' ) {
                        $styles['italic'] = true;
                    }
                    break;
                case 'border-left-color':
                    $styles['borderLeftColor'] = trim($cValue, "#");
                    break;
                case 'border-left-style':
                    $styles['borderLeftStyle'] = $transformBorderStyle($cValue, $styles['borderLeftStyle']);
                    break;
                case 'border-left-width':
                    $styles['borderLeftWidth'] = intval($cValue) * 8;
                    if ( empty($styles['borderLeftWidth']) ) {
                        $styles['borderLeftStyle'] = 'none';
                    }
                    break;
                case 'border-top-color':
                    $styles['borderTopColor'] = trim($cValue, "#");
                    break;
                case 'border-top-style':
                    $styles['borderTopStyle'] = $transformBorderStyle($cValue, $styles['borderTopStyle']);
                    break;
                case 'border-top-width':
                    $styles['borderTopWidth'] = intval($cValue) * 8;
                    if ( empty($styles['borderTopWidth']) ) {
                        $styles['borderTopStyle'] = 'none';
                    }
                    break;
                case 'border-right-color':
                    $styles['borderRightColor'] = trim($cValue, "#");
                    break;
                case 'border-right-style':
                    $styles['borderRightStyle'] = $transformBorderStyle($cValue, $styles['borderRightStyle']);
                    break;
                case 'border-right-width':
                    $styles['borderRightWidth'] = intval($cValue) * 8;
                    if ( empty($styles['borderRightWidth']) ) {
                        $styles['borderRightStyle'] = 'none';
                    }
                    break;
                case 'border-bottom-color':
                    $styles['borderBottomColor'] = trim($cValue, "#");
                    break;
                case 'border-bottom-style':
                    $styles['borderBottomStyle'] = $transformBorderStyle($cValue, $styles['borderBottomStyle']);
                    break;
                case 'border-bottom-width':
                    $styles['borderBottomWidth'] = intval($cValue) * 8;
                    if ( empty($styles['borderBottomWidth']) ) {
                        $styles['borderBottomStyle'] = 'none';
                    }
                    break;
                case 'padding-top':
                    $styles['spaceBefore'] = Converter::pointToTwip($cValue);
                    break;
                case 'padding-bottom':
                    $styles['spaceAfter'] = Converter::pointToTwip($cValue);
                    break;
                case 'line-height':
                    $styles['lineHeight'] = floatval($cValue);
                    break;
                case 'vertical-align':
                    if ($cValue === 'middle') {
                        $styles['VAlign'] = 'center';
                    } else {
                        $styles['VAlign'] = $cValue;
                    }
                    break;
            }
        }

        return $styles;
    }

    /**
     * Parse anchor node
     *
     * @param \DOMNode $node
     * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
     * @return null
     *
     **/
    private static function parseAnchor($node, $element, &$styles)
    {
        $styles['paragraph'] = static::parseInlineStyle($node, $styles['paragraph']);
        $styles['font'] = static::parseInlineStyle($node, $styles['font']);

        // setup default font style
        if (!array_key_exists('color', $styles['font'])) {
            $styles['font']['color'] = "0563C1";
        }
        if (!array_key_exists('underline', $styles['font'])) {
            $styles['font']['underline'] = "single";
        }

        $href = '';
        $newElement = null;
        foreach ($node->attributes as $attribute) {
            if ($attribute->name === 'href') {
                $href = $attribute->value;
            }
        }

        if (!empty($href)) {
            $internal = $href[0] === '#';
            if ($internal) {
                $href = substr($href, 1);
            }
            $newElement = $element->addLink($href, htmlspecialchars($node->nodeValue), $styles['font'], $styles['paragraph'], $internal);
        }
        return $newElement;
    }

    /**
     * Parse horizontal line
     *
     * @param \DOMNode $node
     * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
     * @return null
     *
     **/
    private static function parseLine($node, $element)
    {
        if ($element instanceof \PhpOffice\PhpWord\Element\TextRun && !($element instanceof \PhpOffice\PhpWord\Element\TOCRun)) {
            $element->getParagraphStyle()->setStyleValue('borderBottomSize', 6);
            $element->getParagraphStyle()->setStyleValue('borderBottomStyle', 'single');
        } else {
            $element->addTextRun(array('borderBottomSize' => 6, 'borderBottomStyle' => 'single'));
        }
        return null;
    }

    /**
     * Parse input, default do nothing
     *
     * @param \DOMNode $node
     * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
     * @return \PhpOffice\PhpWord\Element
     *
     **/
    protected static function parseInput($node, $element)
    {
        return $element;
    }

    /**
     * Parse image node
     *
     * @param \DOMNode $node
     * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
     * @return \PhpOffice\PhpWord\Element\TextRun
     *
     **/
    private static function parseImage($node, $element, &$styles)
    {
        $style=array();
        foreach ($node->attributes as $attribute) {
            switch ($attribute->name) {
            case 'src':
                $src=$attribute->value;
                break;
            case 'width':
                $width=$attribute->value;
                $style['width']=$width;
                break;
            case 'height':
                $height=$attribute->value;
                $style['height']=$height;
                break;
            case 'style':
                $styleattr=explode(';', $attribute->value);
                foreach ($styleattr as $attr) {
                    if (strpos($attr, ':')) {
                        list($k, $v) = explode(':', $attr);
                        switch ($k) {
                        case 'float':
                            if (trim($v)=='right') {
                                $style['hPos']=\PhpOffice\PhpWord\Style\Image::POS_RIGHT;
                                $style['hPosRelTo']=\PhpOffice\PhpWord\Style\Image::POS_RELTO_PAGE;
                                $style['pos']=\PhpOffice\PhpWord\Style\Image::POS_RELATIVE;
                                $style['wrap']=\PhpOffice\PhpWord\Style\Image::WRAP_TIGHT;
                                $style['overlap']=true;
                            }
                            if (trim($v)=='left') {
                                $style['hPos']=\PhpOffice\PhpWord\Style\Image::POS_LEFT;
                                $style['hPosRelTo']=\PhpOffice\PhpWord\Style\Image::POS_RELTO_PAGE;
                                $style['pos']=\PhpOffice\PhpWord\Style\Image::POS_RELATIVE;
                                $style['wrap']=\PhpOffice\PhpWord\Style\Image::WRAP_TIGHT;
                                $style['overlap']=true;
                            }
                            break;
                        }
                    }
                }
                break;
            }
        }
        $newElement = $element->addImage($src, $style);
        return $newElement;
    }

    /**
     * Parse span
     *
     * Changes the inline style when a Span element is found.
     *
     * @param type $node
     * @param type $element
     * @param array $styles
     * @return type
     **/
    protected static function parseSpan($node, $element, &$styles)
    {
        $styles['font'] = static::parseInlineStyle($node, $styles['font']);
        return null;
    }

    /**
     * Parse into a nav
     * currently use for table of contents
     */
    private static function parseNav($element, &$data)
    {
        // TODO check if it contains toc
        $newElement = $element->addTOCRun();

        return $newElement;
    }

    private static function parseRevision($node, &$styles)
    {
        global $comment;

        // count($comment) is used as revision ID
        // Push a fake annotation and set its type to 1 (suggestion) so that it will not be exported
        $comment[] = array('type' => 1);

        $c = $comment[$node->getAttribute('data-syno-comment')];
        $styles['font']['revision'] = array(
            'op' => $node->nodeName,
            'id' => $c['id'],
            '@id' => count($comment),
            'author' => $c['author'],
            'date' => $c['date']);
        return null;
    }

    private static function getHeaderFooterType($node)
    {
        $type = $node->getAttribute("type");
        if ($type === "normal") {
            $type = "default";
        } else if ($type === "cover") {
            $type = "first";
        }
        return $type;
    }

    /**
     * Parse header node
     *
     * @param \DOMNode $node
     * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
     * @param array &$styles
     * @return \PhpOffice\PhpWord\Element\Header
     **/
    protected static function parseHeader($node, $element, &$styles)
    {
        $type = self::getHeaderFooterType($node);
        return $type ? $element->addHeader($type) : null;
    }

    /**
     * Parse footer node
     *
     * @param \DOMNode $node
     * @param \PhpOffice\PhpWord\Element\AbstractContainer $element
     * @param array &$styles
     * @return \PhpOffice\PhpWord\Element\Footer
     **/
    protected static function parseFooter($node, $element, &$styles)
    {
        $type = self::getHeaderFooterType($node);
        return $type ? $element->addFooter($type) : null;
    }
}
