<?php
/**
 * This file is part of PHPWord - A pure PHP library for reading and writing
 * word processing documents.
 *
 * PHPWord is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPWord/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPWord
 * @copyright   2010-2014 PHPWord contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpWord\Style;

use PhpOffice\PhpWord\Exception\InvalidStyleException;
use PhpOffice\PhpWord\Shared\CommonString;
use PhpOffice\PhpWord\Style;

/**
 * Paragraph style
 *
 * OOXML:
 * - General: alignment, outline level
 * - Indentation: left, right, firstline, hanging
 * - Spacing: before, after, line spacing
 * - Pagination: widow control, keep next, keep line, page break before
 * - Formatting exception: suppress line numbers, don't hyphenate
 * - Textbox options
 * - Tabs
 * - Shading
 * - Borders
 *
 * OpenOffice:
 * - Indents & spacing
 * - Alignment
 * - Text flow
 * - Outline & numbering
 * - Tabs
 * - Dropcaps
 * - Tabs
 * - Borders
 * - Background
 *
 * @link http://www.schemacentral.com/sc/ooxml/t-w_CT_PPr.html
 */
class Paragraph extends Border
{
    /**
     * @const int One line height equals 240 twip
     */
    const LINE_HEIGHT = 240;

    /**
     * Aliases
     *
     * @var array
     */
    protected $aliases = array('line-height' => 'lineHeight');

    /**
     * Style for next paragraph
     *
     * @var string
     */
    private $next;

    /**
     * Alignment
     *
     * @var \PhpOffice\PhpWord\Style\Alignment
     */
    private $alignment;

    /**
     * Indentation
     *
     * @var \PhpOffice\PhpWord\Style\Indentation
     */
    private $indentation;

    /**
     * Spacing
     *
     * @var \PhpOffice\PhpWord\Style\Spacing
     */
    private $spacing;

    /**
     * Text line height
     *
     * @var int
     */
    private $lineHeight;

    /**
     * Outline level
     *
     * @var int
     */
    private $outlineLevel;

    /**
     * Allow first/last line to display on a separate page
     *
     * @var bool
     */
    private $widowControl = null;

    /**
     * Keep paragraph with next paragraph
     *
     * @var bool
     */
    private $keepNext = null;

    /**
     * Keep all lines on one page
     *
     * @var bool
     */
    private $keepLines = null;

    /**
     * Start paragraph on next page
     *
     * @var bool
     */
    private $pageBreakBefore = null;

    /**
     * Numbering style name
     *
     * @var string
     */
    private $numStyle;

    /**
     * Numbering level
     *
     * @var int
     */
    private $numLevel = null;

    /**
     * Set of Custom Tab Stops
     *
     * @var \PhpOffice\PhpWord\Style\Tab[]
     */
    private $tabs = array();

    /**
     * Shading
     *
     * @var \PhpOffice\PhpWord\Style\Shading
     */
    private $shading;

    /**
     * Font size
     *
     * @var int|float
     */
    private $size;

    /**
     * Font color
     *
     * @var string
     */
    private $color;

    /**
     * Syno Doc ListItem marker
     *
     * @var array
     */
    protected $synoListItem;

    /**
     * Create new instance
     */
    public function __construct()
    {
        $this->alignment = new Alignment();
    }

    /**
     * Set Style value
     *
     * @param string $key
     * @param mixed $value
     * @return self
     */
    public function setStyleValue($key, $value)
    {
        $key = CommonString::removeUnderscorePrefix($key);
        if ($key == 'indent' || $key == 'hanging' || $key == 'firstLine') {
            $value = $value * 720;
        }

        return parent::setStyleValue($key, $value);
    }

    /**
     * Get style values
     *
     * An experiment to retrieve all style values in one function. This will
     * reduce function call and increase cohesion between functions. Should be
     * implemented in all styles.
     *
     * @ignoreScrutinizerPatch
     * @return array
     */
    public function getStyleValues()
    {
        $styles = array(
            'name'              => $this->getStyleName(),
            'basedOn'           => $this->getBasedOn(),
            'next'              => $this->getNext(),
            'alignment'         => $this->getAlign(),
            'indentation'       => $this->getIndentation(),
            'spacing'           => $this->getSpace(),
            'outlineLevel'      => $this->getOutlineLevel(),
            'color'             => $this->getColor(),
            'size'              => $this->getSize(),
            'pagination'        => array(
                'widowControl'  => $this->hasWidowControl(),
                'keepNext'      => $this->isKeepNext(),
                'keepLines'     => $this->isKeepLines(),
                'pageBreak'     => $this->hasPageBreakBefore(),
            ),
            'numbering'         => array(
                'style'         => $this->getNumStyle(),
                'level'         => $this->getNumLevel(),
            ),
            'tabs'              => $this->getTabs(),
            'shading'           => $this->getShading(),
            'revision'          => $this->getRevision(),
            'synoListItem'      => $this->getSynoListItem(),
        );

        return $styles;
    }

    /**
     * Get finalized style
     *
     * @return self
     */
    public function getFinalStyle()
    {
        $chain = $this->getInheritanceChain();
        if (Style::getDefaultParagraphStyle() !== null) {
            array_unshift($chain, Style::getDefaultParagraphStyle());
        }
        foreach ($chain as &$style) {
            if ( get_class($style) != get_class($this) ) {
                $temp = $style->getParagraphStyle();
                $temp->setSize($style->getSize());
                $temp->setColor($style->getColor());
                $style = $temp;
            }
        }
        $result = new self();
        $result->setWidowControl(true);
        for ( $i = 0 ; isset($chain[$i]) ; $i++ ) {
            $values = $chain[$i]->getStyleValues();
            if ( !is_null($values['next']) ) $result->setNext($values['next']);
            if ( !is_null($values['alignment']) ) $result->setAlign($values['alignment']);
            if ( !is_null($values['indentation']) ) $result->setIndentation($values['indentation']);
            if ( !is_null($values['spacing']) ) $result->setSpace($values['spacing']);
            // Note: empty("0") === true
            if ( !is_null($values['outlineLevel']) ) $result->setOutlineLevel($values['outlineLevel']);
            if ( !is_null($values['color']) ) $result->setColor($values['color']);
            if ( !is_null($values['size']) ) $result->setSize($values['size']);
            if ( !is_null($values['tabs']) ) $result->setTabs($values['tabs']);
            if ( !is_null($values['shading']) ) $result->setShading($values['shading']);
            if ( !is_null($values['revision']) ) $result->setRevision($values['revision']);
            // FIXME: synoListItem is a font style and hereby inheritable
            if ( !empty($values['synoListItem']) ) $result->setSynoListItem($values['synoListItem']);

            $pnValues = $values['pagination'];
            if ( !is_null($pnValues['windowControl']) ) $result->setWidowControl($pnValues['windowControl']);
            if ( !is_null($pnValues['keepNext']) ) $result->setKeepNext($pnValues['keepNext']);
            if ( !is_null($pnValues['keepLines']) ) $result->setKeepLines($pnValues['keepLines']);
            if ( !is_null($pnValues['pageBreak']) ) $result->setPageBreakBefore($pnValues['pageBreak']);

            $numValues = $values['numbering'];
            if ( !is_null($numValues['style']) ) $result->setNumStyle($numValues['style']);
            if ( !is_null($numValues['level']) ) $result->setNumLevel($numValues['level']);

            // TODO: border style
        }
        return $result;
    }

    /**
     * Get alignment
     *
     * @return string
     */
    public function getAlign()
    {
        return $this->alignment->getValue();
    }

    /**
     * Set alignment
     *
     * @param string $value
     * @return self
     */
    public function setAlign($value = null)
    {
        $this->alignment->setValue($value);

        return $this;
    }

    /**
     * Get style for next paragraph
     *
     * @return string
     */
    public function getNext()
    {
        return $this->next;
    }

    /**
     * Set style for next paragraph
     *
     * @param string $value
     * @return self
     */
    public function setNext($value = null)
    {
        $this->next = $value;

        return $this;
    }

    /**
     * Get indentation
     *
     * @return \PhpOffice\PhpWord\Style\Indentation
     */
    public function getIndentation()
    {
        return $this->indentation;
    }

    /**
     * Set indentation
     *
     * @param mixed $value
     * @return self
     */
    public function setIndentation($value = null)
    {
        $this->setObjectVal($value, 'Indentation', $this->indentation);

        return $this;
    }

    /**
     * Get left indentation
     *
     * @return int
     */
    public function getIndent()
    {
        return $this->getChildStyleValue($this->indentation, 'left');
    }

    /**
     * Set left indentation
     *
     * @param int $value
     * @return self
     */
    public function setIndent($value = null)
    {
        return $this->setIndentation(array('left' => $value));
    }

    /**
     * Get first line indentation
     *
     * @return int
     */
    public function getFirstLine()
    {
        return $this->getChildStyleValue($this->indentation, 'firstLine');
    }

    /**
     * Set first line indentation
     *
     * @param int $value
     * @return self
     */
    public function setFirstLine($value = null)
    {
        return $this->setIndentation(array('firstLine' => $value));
    }

    /**
     * Get hanging
     *
     * @return int
     */
    public function getHanging()
    {
        return $this->getChildStyleValue($this->indentation, 'hanging');
    }

    /**
     * Set hanging
     *
     * @param int $value
     * @return self
     */
    public function setHanging($value = null)
    {
        return $this->setIndentation(array('hanging' => $value));
    }

    /**
     * Get spacing
     *
     * @return \PhpOffice\PhpWord\Style\Spacing
     * @todo Rename to getSpacing in 1.0
     */
    public function getSpace()
    {
        return $this->spacing;
    }

    /**
     * Set spacing
     *
     * @param mixed $value
     * @return self
     * @todo Rename to setSpacing in 1.0
     */
    public function setSpace($value = null)
    {
        $this->setObjectVal($value, 'Spacing', $this->spacing);

        return $this;
    }

    /**
     * Get outline level
     *
     * @return int
     */
    public function getOutlineLevel()
    {
        return $this->outlineLevel;
    }

    /**
     * Set outline level
     *
     * @param int
     * @return self
     */
    public function setOutlineLevel($value = null)
    {
        $this->outlineLevel = $value;

        return $this;
    }

    /**
     * Get space before paragraph
     *
     * @return integer
     */
    public function getSpaceBefore()
    {
        return $this->getChildStyleValue($this->spacing, 'before');
    }

    /**
     * Set space before paragraph
     *
     * @param int $value
     * @return self
     */
    public function setSpaceBefore($value = null)
    {
        return $this->setSpace(array('before' => $value));
    }

    /**
     * Get space after paragraph
     *
     * @return integer
     */
    public function getSpaceAfter()
    {
        return $this->getChildStyleValue($this->spacing, 'after');
    }

    /**
     * Set space after paragraph
     *
     * @param int $value
     * @return self
     */
    public function setSpaceAfter($value = null)
    {
        return $this->setSpace(array('after' => $value));
    }

    /**
     * Get spacing between lines
     *
     * @return int
     */
    public function getSpacing()
    {
        return $this->getChildStyleValue($this->spacing, 'line');
    }

    /**
     * Set spacing between lines
     *
     * @param int $value
     * @return self
     */
    public function setSpacing($value = null)
    {
        return $this->setSpace(array('line' => $value));
    }

    /**
     * Get line height
     *
     * @return int|float
     */
    public function getLineHeight()
    {
        return $this->lineHeight;
    }

    /**
     * Set the line height
     *
     * @param int|float|string $lineHeight
     * @return self
     * @throws \PhpOffice\PhpWord\Exception\InvalidStyleException
     */
    public function setLineHeight($lineHeight)
    {
        if (is_string($lineHeight)) {
            $lineHeight = floatval(preg_replace('/[^0-9\.\,]/', '', $lineHeight));
        }

        if ((!is_integer($lineHeight) && !is_float($lineHeight)) || !$lineHeight) {
            throw new InvalidStyleException('Line height must be a valid number');
        }

        $this->lineHeight = $lineHeight;
        $this->setSpacing($lineHeight * self::LINE_HEIGHT);
        return $this;
    }

    /**
     * Get allow first/last line to display on a separate page setting
     *
     * @return bool
     */
    public function hasWidowControl()
    {
        return $this->widowControl;
    }

    /**
     * Set keep paragraph with next paragraph setting
     *
     * @param bool $value
     * @return self
     */
    public function setWidowControl($value = true)
    {
        $this->widowControl = $this->setBoolVal($value, $this->widowControl);

        return $this;
    }

    /**
     * Get keep paragraph with next paragraph setting
     *
     * @return bool
     */
    public function isKeepNext()
    {
        return $this->keepNext;
    }

    /**
     * Set keep paragraph with next paragraph setting
     *
     * @param bool $value
     * @return self
     */
    public function setKeepNext($value = true)
    {
        $this->keepNext = $this->setBoolVal($value, $this->keepNext);

        return $this;
    }

    /**
     * Get keep all lines on one page setting
     *
     * @return bool
     */
    public function isKeepLines()
    {
        return $this->keepLines;
    }

    /**
     * Set keep all lines on one page setting
     *
     * @param bool $value
     * @return self
     */
    public function setKeepLines($value = true)
    {
        $this->keepLines = $this->setBoolVal($value, $this->keepLines);

        return $this;
    }

    /**
     * Get start paragraph on next page setting
     *
     * @return bool
     */
    public function hasPageBreakBefore()
    {
        return $this->pageBreakBefore;
    }

    /**
     * Set start paragraph on next page setting
     *
     * @param bool $value
     * @return self
     */
    public function setPageBreakBefore($value = true)
    {
        $this->pageBreakBefore = $this->setBoolVal($value, $this->pageBreakBefore);

        return $this;
    }

    /**
     * Get numbering style name
     *
     * @return string
     */
    public function getNumStyle()
    {
        return $this->numStyle;
    }

    /**
     * Set numbering style name
     *
     * @param string $value
     * @return self
     */
    public function setNumStyle($value)
    {
        $this->numStyle = $value;

        return $this;
    }

    /**
     * Get numbering level
     *
     * @return int
     */
    public function getNumLevel()
    {
        return $this->numLevel;
    }

    /**
     * Set numbering level
     *
     * @param int $value
     * @return self
     */
    public function setNumLevel($value = 0)
    {
        $this->numLevel = $this->setIntVal($value, $this->numLevel);

        return $this;
    }

    /**
     * Get tabs
     *
     * @return \PhpOffice\PhpWord\Style\Tab[]
     */
    public function getTabs()
    {
        return $this->tabs;
    }

    /**
     * Set tabs
     *
     * @param array $value
     * @return self
     */
    public function setTabs($value = null)
    {
        if (is_array($value)) {
            $this->tabs = $value;
        }

        return $this;
    }

    /**
     * Get allow first/last line to display on a separate page setting
     *
     * @deprecated 0.10.0
     * @codeCoverageIgnore
     */
    public function getWidowControl()
    {
        return $this->hasWidowControl();
    }

    /**
     * Get keep paragraph with next paragraph setting
     *
     * @deprecated 0.10.0
     * @codeCoverageIgnore
     */
    public function getKeepNext()
    {
        return $this->isKeepNext();
    }

    /**
     * Get keep all lines on one page setting
     *
     * @deprecated 0.10.0
     * @codeCoverageIgnore
     */
    public function getKeepLines()
    {
        return $this->isKeepLines();
    }

    /**
     * Get start paragraph on next page setting
     *
     * @deprecated 0.10.0
     * @codeCoverageIgnore
     */
    public function getPageBreakBefore()
    {
        return $this->hasPageBreakBefore();
    }

    /**
     * Get shading
     *
     * @return \PhpOffice\PhpWord\Style\Shading
     */
    public function getShading()
    {
        return $this->shading;
    }

    /**
     * Set shading
     *
     * @param mixed $value
     * @return self
     */
    public function setShading($value = null)
    {
        $this->setObjectVal($value, 'Shading', $this->shading);

        return $this;
    }

    /**
     * Get font size
     *
     * @return int|float
     */
    public function getSize()
    {
        return $this->size;
    }

    /**
     * Set font size
     *
     * @param int|float $value
     * @return self
     */
    public function setSize($value = null)
    {
        $this->size = $this->setNumericVal($value, $this->size);

        return $this;
    }

    /**
     * Get font color
     *
     * @return string
     */
    public function getColor()
    {
        return $this->color;
    }

    /**
     * Set font color
     *
     * @param string $value
     * @return self
     */
    public function setColor($value = null)
    {
        $this->color = $value;

        return $this;
    }

    protected $revision = null;

    public function setRevision($revision)
    {
        if (!is_array($revision)) {
            return;
        }
        $this->revision = $revision;
    }

    public function getRevision()
    {
        return $this->revision;
    }

    public function setSynoListItem($synoListItem = null)
    {
        if (!is_array($synoListItem)) {
            return;
        }
        $this->synoListItem = $synoListItem;
    }

    public function getSynoListItem()
    {
        return $this->synoListItem;
    }
}
